/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This file was a part of IonShell (https://gitlab.com/cubocore/ion/ionde)
	* Suitable modifications have been done to meet the needs of DesQ.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QGuiApplication>
#include <QWindow>
#include <QScreen>

#include <qpa/qplatformnativeinterface.h>
#include <wayland-client.h>

#include "DesQWlGlobal.hpp"
#include "LayerShell.hpp"
#include "Registry.hpp"

#include "qwayland-wlr-layer-shell-unstable-v1.h"

wl_display* DesQWayland::getWlDisplay() {

	QPlatformNativeInterface *native = QGuiApplication::platformNativeInterface();
	if ( !native )
		return nullptr;

	struct wl_display *display = static_cast<struct ::wl_display *>( native->nativeResourceForWindow( "display", nullptr ) );

	return display;
};

wl_output* DesQWayland::getWlOutput( QScreen *screen ) {

	QPlatformNativeInterface *native = QGuiApplication::platformNativeInterface();
	if ( !native )
		return nullptr;

	struct wl_output *output = static_cast<struct ::wl_output*>( native->nativeResourceForScreen( "output", screen ) );

	return output;
};

wl_surface* DesQWayland::getWlSurface( QWindow *window ) {

	QPlatformNativeInterface *native = QGuiApplication::platformNativeInterface();
	if ( !native )
		return nullptr;

	struct wl_surface *surface = static_cast<struct ::wl_surface *>( native->nativeResourceForWindow( "surface", window ) );

	return surface;
};

bool DesQWayland::isRunning() {

	/* Check if XDG_SESSION_TYPE is set */
	QString session = qgetenv( "XDG_SESSION_TYPE" );
	if ( session.toLower() == QStringLiteral( "wayland" ) )
		return true;

	/*
		* May be XDG_SESSION_TYPE is not set. Try Harder.
		* We check if WAYLAND_DISPLAY is set.
	*/
	QString wlID = qgetenv( "WAYLAND_DISPLAY" );
	if ( not wlID.isEmpty() )
		return true;

	/*
		* May be WAYLAND_DISPLAY is not set. Try Harder still.
		* We check if DESKTOP_SESSION is set.
	*/
	QString deSess = qgetenv( "DESKTOP_SESSION" );
	if ( deSess.contains( "wayland-sessions" ) )
		return true;

	/* It's probably not a wayland session. */
	return false;
};
