/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QMargins>
#include <QSize>

class QWindow;

#include "LayerShell.hpp"
#include "qwayland-wlr-layer-shell-unstable-v1.h"

class LayerShellPrivate : public QtWayland::zwlr_layer_shell_v1 {

	public:
		LayerShellPrivate( struct ::zwlr_layer_shell_v1 *lShell, LayerShell *ptr );
		struct ::zwlr_layer_surface_v1 *getLayerSurface(QWindow *window, LayerShell::LayerType layer, const QString &layer_namespace);

	private:
		struct ::zwlr_layer_shell_v1 *shellPtr;

	protected:
		Q_DISABLE_COPY( LayerShellPrivate )
		Q_DECLARE_PUBLIC( LayerShell )

		LayerShell *q_ptr;
};

class LayerSurfacePrivate : public QtWayland::zwlr_layer_surface_v1 {

	public:
		explicit LayerSurfacePrivate( QWindow *window, LayerShell *lShell, LayerShell::LayerType type, LayerSurface *ptr );
		void apply();

	protected:
		void zwlr_layer_surface_v1_configure(uint32_t serial, uint32_t width, uint32_t height) override;
		void zwlr_layer_surface_v1_closed() override;

		Q_DISABLE_COPY( LayerSurfacePrivate )
		Q_DECLARE_PUBLIC( LayerSurface )

		LayerSurface *q_ptr;

	private:
		QWindow *m_window;

		QSize m_surfaceSize;
		LayerSurface::SurfaceAnchors m_anchors;
		int m_exclusiveZone;
		QMargins m_margins;
		uint m_keyboardInteractivity;

		QSize m_surfacePendingSize;
		bool isConfigured;
};
