/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This file was a part of IonShell (https://gitlab.com/cubocore/ion/ionde)
	* Suitable modifications have been done to meet the needs of DesQ.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>

#include "Registry.hpp"
#include "RegistryImpl.hpp"

#include "LayerShell.hpp"

#include "qwayland-wayfire-shell-unstable-v2.h"
#include "qwayland-wlr-layer-shell-unstable-v1.h"
#include "qwayland-wlr-foreign-toplevel-management-unstable-v1.h"

WaylandRegistryPrivate::WaylandRegistryPrivate( wl_display *wlDisplay, WaylandRegistry *ptr ) : q_ptr( ptr ), m_wlDisplay( wlDisplay ) {
}

WaylandRegistryPrivate::~WaylandRegistryPrivate() {

	wl_registry_destroy( object() );
}

void WaylandRegistryPrivate::initialize() {

	init( wl_display_get_registry( m_wlDisplay ) );
	wl_display_roundtrip( m_wlDisplay ); // Wait until the server responds
}

void WaylandRegistryPrivate::registry_global( uint32_t name, const QString &interface, uint32_t version ) {

	Q_Q( WaylandRegistry );

	if ( interface == wl_compositor_interface.name ) {
		m_wlCompositor = static_cast<wl_compositor*>( bind( name, &wl_compositor_interface, version ) );

		if (!m_wlCompositor)
			emit q->errorOccured( WaylandRegistry::EmptyCompositor );
	}

	else if ( interface == wl_seat_interface.name ) {
		m_wlSeat = static_cast<wl_seat*>( bind( name, &wl_seat_interface, version ) );

		if (!m_wlSeat)
			emit q->errorOccured( WaylandRegistry::EmptySeat );
	}

	else if (interface == zwlr_layer_shell_v1_interface.name) {
		m_wlrLayerShell = static_cast<zwlr_layer_shell_v1*>( bind( name, &zwlr_layer_shell_v1_interface, 1 ) );

		if (!m_wlrLayerShell)
			emit q->errorOccured( WaylandRegistry::EmptyLayerShell );

		else
			m_layerShell = new LayerShell( m_wlrLayerShell );
	}

	else if (interface == zwf_shell_manager_v2_interface.name) {
		m_wayfireShell = static_cast<zwf_shell_manager_v2*>(bind(name, &zwf_shell_manager_v2_interface, 1));
	}
}

void WaylandRegistryPrivate::registry_global_remove(uint32_t name) {
}

/* ============================================================================ */

WaylandRegistry::WaylandRegistry( wl_display *display ) : QObject() {

	d_ptr = new WaylandRegistryPrivate( display, this );
}

WaylandRegistry::~WaylandRegistry() {

}

void WaylandRegistry::initialize() {

	Q_D( WaylandRegistry );
	d->initialize();
}

wl_display *WaylandRegistry::wlDisplay() const {

	Q_D( const WaylandRegistry );
	return d->m_wlDisplay;
}

wl_seat *WaylandRegistry::wlSeat() const {

	Q_D( const WaylandRegistry );
	return d->m_wlSeat;
}

wl_compositor *WaylandRegistry::wlCompositor() const {

	Q_D( const WaylandRegistry );
	return d->m_wlCompositor;
}

zwlr_layer_shell_v1 *WaylandRegistry::wlrLayerShell() const {

	Q_D( const WaylandRegistry );
	return d->m_wlrLayerShell;
}

zwlr_foreign_toplevel_manager_v1 *WaylandRegistry::wlrToplevelManager() const {

	Q_D( const WaylandRegistry );
	return d->m_wlrToplevelManager;
}

zwf_shell_manager_v2 *WaylandRegistry::wayfireShell() const {

	Q_D( const WaylandRegistry );
	return d->m_wayfireShell;
}

LayerShell *WaylandRegistry::lShell() const {

	Q_D( const WaylandRegistry );
	return d->m_layerShell;
}

void WaylandRegistry::registry_global( uint32_t name, const QString &interface, uint32_t version ) {

	Q_D( WaylandRegistry );
	d->registry_global( name, interface, version );
}

void WaylandRegistry::registry_global_remove( uint32_t name ) {

	Q_D( WaylandRegistry );
	d->registry_global_remove( name );
}
