/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This file was a part of IonShell (https://gitlab.com/cubocore/ion/ionde)
	* Suitable modifications have been done to meet the needs of DesQ.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QThread>

struct wl_seat;
struct wl_display;
struct wl_compositor;
struct zwlr_layer_shell_v1;
struct zwf_shell_manager_v2;
struct zwlr_foreign_toplevel_manager_v1;

class WaylandRegistryPrivate;
class LayerShell;

class WaylandRegistry : public QObject {
	Q_OBJECT

	public:
		enum ErrorType {
			EmptyWayfireShell,
			EmptyCompositor,
			EmptySeat,
			EmptyLayerShell,
			EmptyToplevelManager
		};

		Q_ENUM(ErrorType)

		WaylandRegistry();
		WaylandRegistry( wl_display *wlDisplay );
		~WaylandRegistry();

		void initialize();

		wl_display *wlDisplay() const;
		wl_seat *wlSeat() const;
		wl_compositor *wlCompositor() const;
		zwlr_layer_shell_v1 *wlrLayerShell() const;
		zwlr_foreign_toplevel_manager_v1 *wlrToplevelManager() const;
		zwf_shell_manager_v2 *wayfireShell() const;

		LayerShell *lShell() const;

	signals:
		void errorOccured( ErrorType et );

	protected:
		void registry_global( uint32_t name, const QString &interface, uint32_t version );
		void registry_global_remove( uint32_t name );

		Q_DECLARE_PRIVATE( WaylandRegistry );
		WaylandRegistryPrivate *d_ptr;
};
